#!/usr/bin/env python
# -*- coding: utf-8 -*-
#   ___      _ _    _   ___     _     ___ _ _
#  | _ )_  _(_) |__| | | _ \___| |_  | __(_) |___
#  | _ \ || | | / _` | |  _/ _ \  _| | _|| | / -_)
#  |___/\_,_|_|_\__,_| |_| \___/\__| |_| |_|_\___|
#
# Internationalisation de projet: python ou bash
#
# Lecture recursive des dossiers depuis l'endroit ou est active bpf.py avec"
# une reconnaissance des fichiers SH, PY et GLADE qui seront traites par xgettext,"
# sed, msgcat,msgmerge et enfin msgfmt. "
# Par defaut, les fichiers de langues fr, en et en_GB seront crees dans le dossier"
# po/ si ils n'existent pas et les autres fichiers de langues seront conserves et"
# mis a jour."
# Au final seront crees les fichiers po, pot et mo correspondants a l'application"
# traitee."
#
# Claude Bulin
# xcfaudio [ at ] gmail [ dot ] com
# http://www.xcfa.tuxfamily.org/
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; version 3.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA
#
# Date    : Thu, 01 Aug 2013 18:46:06
# Author  : xcfaudio
#
# $ figlet -w 200 -f small " Build Pot File"
#


import os, sys, subprocess, gettext, datetime

# FOR COMMENT
t = gettext.translation('bpf', '/usr/share/locale', fallback=True)
_ = t.gettext

try:
	import polib
except:
	if sys.stdout.isatty() == True:
		print( "")
		print( "bpf (Build Pot File) ")
		print( ">>>  " + _("Please install python-polib module") + "  <<<" )
		print( "")
	exit(0)

import urllib
import urllib.request
import http.client
import socket

version_package = "1.4"
name_package = "bpf (Build Pot File) " + version_package

try:
	import argparse
except:
	if sys.stdout.isatty() == True:
		print( "")
		print( "bpf (Build Pot File) ")
		print( ">>>  " + _("Please install python-argparse module") + "  <<<" )
		print( "")
	exit(0)


langage_folder_path = os.getcwd()
path_po             = os.path.join(langage_folder_path, 'po')
path_po_pot         = os.path.join(langage_folder_path, 'po/pot')


tab_ui = []				# List ui file
tab_py = []				# List py file
tab_glade = []			# List glade file
tab_sh = []				# List sh file
nbr_rep_po = 0			#
ListPotFile=[]			# List add pot file(s)

Bool_CompletionTranslate = False
Bool_ForceTranslate = False


if sys.stdout.isatty() == False:
	if os.path.isfile("/usr/bin/zenity") == True:
		command = 'zenity --info --text="Run script only from a terminal\n\nchmod +x ./bpf.py\n./bpf.py\nor\nbpf.py" --title="Build Pot File"'
		subprocess.call(command, shell=True)
	else:
		command = "vte --command 'dialog --title \"Build Pot File\" --msgbox \"\nRun script only from a terminal\n\nchmod +x ./bpf.py\n./bpf.py\nor\nbpf.py\" 12 50'"
		subprocess.call(command, shell=True)
	exit(0)


def print_entete():
	print( "!----------------------" )
	print( "! " )
	print( "! %s" %name_package )
	print( "!", _("Internationalization of project: Python or Bash.") )
	print( "! " )
	print( "!----------------------" )
	print( _("Read recursive folders with recognition files Bash, Python, Glade will be treated with xgettext, sed, msgcat, msgmerge and msgfmt.") )
	print( _("By default, the language files fr, en and en_GB folders will be created in the PO/ if they do not exist and other language files will be kept and updated.") )
	print( _("An option allows the automatic translation of msgstr fields.") )
	print( _("The final will be created corresponding files po, mo pot application processed.") )
	print()


def test_arguments():

	global ListPotFile
	global Bool_CompletionTranslate
	global Bool_ForceTranslate

	bool_no_arg = False
	if len(sys.argv) <= 1:
		bool_no_arg = True

	parser = argparse.ArgumentParser(description=name_package)
	parser.add_argument("-a", "--AddPotFile", help='add *.po files from country1.po[,country2.po,...]. See the manpage for countries', type=str )
	parser.add_argument("-c", "--CompletionTranslate", help='Complete the missing translations to "msgstr"', default=False, action='store_true', required=False)
	parser.add_argument("-f", "--ForceTranslate", help='Force the translation of all fields to "msgstr"', default=False, action='store_true', required=False)
	parser.add_argument("-g", "--generate", help='Build files *.po, *.mo and *.pot ', action='store_true', required=False)
	parser.add_argument("-v", "--version",  help='Version is '+version_package,  action='store_true', required=False)
	args = parser.parse_args()

	if args.generate:
		#print( " "*4,"Generation OK" )
		print( "" )

	if args.version :
		print( "" )
		print( _("Version is:") )
		print( "  ",name_package)
		print( "" )
		exit(0)

	if bool_no_arg == True:
		parser.print_help()
		print("")
		exit(0)

	if args.AddPotFile :
		ListPotFile = args.AddPotFile.split(",")

	if not "fr.po" in ListPotFile:
		ListPotFile.append( 'fr.po' )
	if not "en.po" in ListPotFile:
		ListPotFile.append( 'en.po' )
	if not "en_GB.po" in ListPotFile:
		ListPotFile.append( 'en_GB.po' )

	if not args.generate:
		if ListPotFile:
			print("")
			print(_("List new file.po for directory po/:"))
			for LN in ListPotFile:
				print("  ", LN)
		print("")
		print(_("Use '-g' option for generate files po"))
		print("")
		exit(0)

	Bool_CompletionTranslate = args.CompletionTranslate
	Bool_ForceTranslate = args.ForceTranslate

## stocke
##   les dossiers: sh, ui, py, glade
##   le nombre de dossiers po/
def test_rep_whith_files( folder_path ):

	global nbr_rep_po
	global tab_ui
	global tab_py
	global tab_glade
	global tab_sh

	for root, dirs, files in os.walk(folder_path):
		if(files):
			for f in files:
				bool_tab_ui = False
				bool_tab_py = False
				bool_tab_glade = False
				bool_tab_sh = False
				fileName, fileExtension = os.path.splitext( f )
				if fileExtension == ".ui":
					bool_tab_ui = True
				elif fileExtension == ".py":
					bool_tab_py = True
				elif fileExtension == ".glade":
					bool_tab_glade = True
				elif fileExtension == ".sh":
					bool_tab_sh = True

				if bool_tab_ui == True:
					tab_ui.append( root )
				if bool_tab_py == True:
					tab_py.append( root )
				if bool_tab_glade == True:
					tab_glade.append( root )
				if bool_tab_sh == True:
					tab_sh.append( root )

		t = root.split('/')
		if "po" == t[ len(t)-1 ] :
			nbr_rep_po+= 1

	print( " "*4,"len(tab_ui)    ", len(tab_ui), "=", tab_ui)
	print( " "*4,"len(tab_py)    ", len(tab_py), "=", tab_py)
	print( " "*4,"len(tab_glade) ", len(tab_glade), "=", tab_glade)
	print( " "*4,"len(tab_sh)    ", len(tab_sh), "=", tab_sh)
	print( " "*4,"nbr_rep_po     ", nbr_rep_po )


def make_xgettext():

	global tab_ui
	global tab_py
	global tab_glade
	global tab_sh

	if tab_ui:
		print( " "*4,_("UI File"))
		Cpt = 1
		for tab in tab_ui:
			os.chdir( tab )
			command = ""
			command = command + "xgettext -c -L Glade --sort-output -o "
			command = command + "'" + path_po_pot + "/ALL_" + str(Cpt) + ".pot' *.ui"
			print( " "*8, command )
			subprocess.call(command, shell=True)
			Cpt += 1

	if tab_py:
		print( " "*4,_("PY File"))
		Cpt = 1
		for tab in tab_py:
			os.chdir( tab )
			command = ""
			command = command + "xgettext --language=Python --from-code=Python --keyword=_ --sort-output -o "
			command = command + "'" + path_po_pot + "/ALL_" + str(Cpt) + ".pot' *.py"
			print( " "*8, command )
			subprocess.call(command, shell=True)
			Cpt += 1

	if tab_glade:
		print( " "*4,_("GLADE File"))
		Cpt = 1
		for tab in tab_glade:
			os.chdir( tab )
			command = ""
			command = command + "xgettext -c -L Glade --sort-output -o "
			command = command + "'" + path_po_pot + "/ALL_" + str(Cpt) + ".pot' *.glade"
			print( " "*8, command )
			subprocess.call(command, shell=True)
			Cpt += 1

	if tab_sh:
		print( " "*4,_("BASH File"))
		Cpt = 1
		for tab in tab_sh:
			os.chdir( tab )
			command = ""
			command = command + "xgettext -c -L Shell --sort-output -o "
			command = command + "'" + path_po_pot + "/ALL_" + str(Cpt) + ".pot' *.sh"
			print( " "*8, command )
			subprocess.call(command, shell=True)
			Cpt += 1

	print( _("Use sed from") + " [ " + path_po_pot + " ]" )
	os.chdir( path_po_pot )
	command = ""
	command = command + "sed -i 's/charset=CHARSET/charset=UTF-8/g' *.pot"
	print( " "*4, command )
	subprocess.call(command, shell=True)


def make_msgcat():
	os.chdir( path_po_pot )
	command = ""
	for root, dirs, files in os.walk(path_po_pot):
		if files:
			for f in files:
				command = command + "./" + f + " "
	if command != "":
		command = command + "-F -o " + "'" + path_po + "/ALL.pot'"
		command = "msgcat " + command
		print( " "*4, command )
		subprocess.call(command, shell=True)
	else:
		print( " "*4,"Aucun fichier *.pot" )


def create_langues():

	global ListPotFile

	# header of POT file
	header_text = ""
	header_text = header_text + '# NAME_PROJECT POT Template File.\n'
	header_text = header_text + '# Copyright (C) 2013  YOUR_NAME\n'
	header_text = header_text + '# This file is distributed under the same license as the PACKAGE package.\n'
	header_text = header_text + '# YOUR_NAME <ADR_MAIL>, 2013.\n'
	header_text = header_text + '#\n'
	header_text = header_text + 'msgid ""\n'
	header_text = header_text + 'msgstr ""\n'
	header_text = header_text + '"Project-Id-Version: NAME_PROJECT (version: XXX)\\n"\n'
	header_text = header_text + '"Report-Msgid-Bugs-To: YOUR_NAME <ADR_MAIL>\\n"\n'
	header_text = header_text + '"POT-Creation-Date: %s\\n"\n' % datetime.datetime.now()
	header_text = header_text + '"PO-Revision-Date: YEAR-MO-DA HO:MI+ZONE\\n"\n'
	header_text = header_text + '"Last-Translator: YOUR_NAME <ADR_MAIL>\\n"\n'
	header_text = header_text + '"Language-Team: URL\\n"\n'
	header_text = header_text + '"MIME-Version: 1.0\\n"\n'
	header_text = header_text + '"Content-Type: text/plain; charset=UTF-8\\n"\n'
	header_text = header_text + '"Content-Transfer-Encoding: 8bit\\n"\n'

	os.chdir( path_po )
	dirs = os.listdir(path_po)
	bool_create = False
	for LN in ListPotFile:
		if not LN in dirs:
			print( " "*4,_("Create"), LN )
			f = open(LN, "w")
			f.write(header_text)
			f.close()
			bool_create = True
	if bool_create == False:
		print( " "*4, _("The files exist") )


def make_msgmerge():
	os.chdir( path_po )
	dirs = os.listdir(path_po)
	for file in dirs:
		if os.path.isfile(file):
			fileName, fileExtension = os.path.splitext( file )
			if ".po" == fileExtension:
				command = "msgmerge --quiet " + file + " ALL.pot -U"
				print( " "*4, command )
				subprocess.call(command, shell=True)


def make_msgfmt():
	os.chdir( path_po )
	dirs = os.listdir(path_po)
	for file in dirs:
		if os.path.isfile(file):
			fileName, fileExtension = os.path.splitext( file )
			if ".po" == fileExtension:
				command = "msgfmt "+ file + " --output-file=" + fileName + ".mo"
				print( " "*4, command )
				subprocess.call(command, shell=True)


#
# From
#	http://stackoverflow.com/questions/17440343/python-checking-internet-connection-more-than-once
# Recoded
#
def test_internet_connection():
	BoolConnected = False
	test_con_url = "www.google.com" # For connection testing
	test_con_resouce = "/intl/en/policies/privacy/" # may change in future
	test_con = http.client.HTTPConnection(test_con_url) # create a connection

	try:
		test_con.request("GET", test_con_resouce) # do a GET request
		response = test_con.getresponse()
	except httplib.ResponseNotReady as e:
		# print( "Improper connection state" )
		BoolConnected = False
	except socket.gaierror as e:
		# print( "Not connected" )
		BoolConnected = False
	else:
		# print( "Connected" )
		BoolConnected = True

	test_con.close()

	return BoolConnected


#
# From
#	https://pypi.python.org/pypi/pytranslate/0.1.3
#	https://github.com/stockrt/pytranslate
# Recoded
#
def get_translating(text, sl='auto', tl='portuguese'):

	BoolErrorNetwork = False;

	urllib.request.FancyURLopener.version = 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.168 Safari/535.19'
	f = urllib.request.FancyURLopener()
	post_params = urllib.parse.urlencode({'client':'t', 'text':text, 'sl':'%s' % sl.lower(), 'tl':'%s' % tl.lower()})
	try:
		content = f.open('http://translate.google.com/translate_a/t', post_params).read()
		Buffer = content.decode()
		begin = Buffer.find( '"' )
		begin += 1
		end   = Buffer.find( '"', begin )
		StringTranslate = ""
		while begin < end:
			StringTranslate += Buffer[ begin ]
			begin += 1
		BoolErrorNetwork = False;
	except:
		StringTranslate = ""
		BoolErrorNetwork = True;

	return BoolErrorNetwork, text, StringTranslate


#
# From
#	https://bitbucket.org/izi/polib/wiki/Home
# Recoded
#
def translate( pathnamefile, BoolForce = False ):

	if test_internet_connection() == False:
		print( " "*4, ">>>  " + _("No network connection") + "  <<<")
		return

	# Get name file without ext
	translate_to = os.path.splitext(os.path.basename(pathnamefile))[0]
	# The first two characters
	translate_to = translate_to[0:2]
	# Open file 'po'
	po = polib.pofile( pathnamefile )
	# Browse file 'po'
	for entry in po:
		# Do not force the translation
		if BoolForce == False:
			if entry.msgstr == "":
				# Delete the carriage returns
				String = entry.msgid.replace( '\n', ' ' )
				# Translate
				BoolErrorNetwork, str_origine, str_trans = get_translating( String, sl='auto', tl=translate_to )
				# Assignment of translation
				entry.msgstr = str_trans
		# Force translation
		else:
			# Delete the carriage returns
			String = entry.msgid.replace( '\n', ' ' )
			# Translate
			BoolErrorNetwork, str_origine, str_trans = get_translating( String, sl='auto', tl=translate_to )
			# Assignment of translation
			entry.msgstr = str_trans
		if BoolErrorNetwork == True:
			print( " "*4, ">>>  " + _("No network connection from google") + "  <<<")
			return

# 	print( "" )
# 	for entry in po:
# 		for occu in entry.occurrences:
# 			print( occu )
# 		print( entry.msgid )
# 		print( entry.msgstr )

	# Save 'po' file with translation
	po.save()

print_entete()

# test privileges
if os.geteuid() == 0:
	print( ">>> [ " + name_package + " ] " + _("should not be run as root") + " <<<" )
	print( "!\n\n" )
	exit(0)

test_arguments()

print( _("Read directories") )
test_rep_whith_files( langage_folder_path )

print( _("Create") + " [ " + path_po_pot + " ]" )
if not os.path.isdir( path_po_pot ):
    os.makedirs( path_po_pot )
    print( " "*4, _("Creating ok") )
else:
    print( " "*4, _("Present already") )

print( _("Use xgettext from") + " [ " + path_po_pot + " ]" )
make_xgettext()

print(  _("Creating file po with msgcat from") + " [ " + path_po + " ]" )
make_msgcat()

print(  _("Creating file po if not exist from") + " [ " + path_po + " ]" )
create_langues()

print(  _("Extracting") + " [ " + path_po + " ] " + _("with msgmerge") )
make_msgmerge()

if Bool_CompletionTranslate == True or Bool_ForceTranslate == True:
	print(  _("Translating files From the network ...") )
	os.chdir( path_po )
	dirs = os.listdir(path_po)
	for file in dirs:
		if os.path.isfile(file):
			fileName, fileExtension = os.path.splitext( file )
			if ".po" == fileExtension:
				print( " "*4, file )
				if Bool_CompletionTranslate == True:
					translate( file, BoolForce = False )
				elif Bool_ForceTranslate == True:
					translate( file, BoolForce = True )

print(  _("Create file(s) MO in the directory") + " [ " + path_po + " ]" )
make_msgfmt()

print(  _("Delete directory") + " [ " + path_po_pot + " ]")
os.chdir( path_po )
command = "rm -rf pot/"
subprocess.call(command, shell=True)

print(  _("Delete of files") + " [ " + path_po + "/*~ ]" )
os.chdir( path_po )
command = "rm -f *~"
subprocess.call(command, shell=True)

print(  _("files produced") + " [ " + path_po + " ]" )
os.chdir( path_po )
dirs = os.listdir(path_po)
for file in dirs:
	if os.path.isfile(file):
		print( " "*4, file )

print("")
print( _("End of treatment.") )
print("")
